==> solverz-0.0.3.tar.gz: solverz-0.0.3/Solverz/solvers/daesolver/trapezoidal.py <==
from Solverz.solvers.daesolver.utilities import *


@dae_io_parser
def implicit_trapezoid(dae: nDAE,
                       tspan: List | np.ndarray,
                       y0: np.ndarray,
                       opt: Opt = None):
    r"""
    The fixed step implicit trapezoidal method.

    Parameters
    ==========

    dae : nDAE
        Numerical DAE object.

    tspan : List | np.ndarray
        An array specifying t0 and tend

    y0 : np.ndarray
        The initial values of variables

    opt : Opt
        The solver options, including:

        - step_size: 1e-3(default)|float
            The step size

    Returns
    =======

    sol : daesol
        The daesol object.


    References
    ==========

    .. [1] https://en.wikipedia.org/wiki/Trapezoidal_rule_(differential_equations)

    """
    stats = Stats(scheme='Trapezoidal')
    if opt is None:
        opt = Opt(stats=True)
    dt = opt.step_size
    tspan = np.array(tspan)
    T_initial = tspan[0]
    T_end = tspan[-1]
    nt = 0
    tt = T_initial
    t0 = tt

    y = np.zeros((10000, y0.shape[0]))
    y0 = DaeIc(dae, y0, t0, opt.rtol)  # check and modify initial values
    y[0, :] = y0
    T = np.zeros((10000,))

    p = dae.p
    while abs(tt - T_end) > abs(dt) / 10:
        My0 = dae.M @ y0
        F0 = dae.F(t0, y0, p).copy()
        ae = nAE(lambda y_, p_: dt / 2 * (dae.F(t0 + dt, y_, p_) + F0) - dae.M @ y_ + My0,
                 lambda y_, p_: -dae.M + dt / 2 * dae.J(t0 + dt, y_, p_),
                 p)

        sol = nr_method(ae, y0, Opt(stats=True))
        y1 = sol.y
        ite = sol.stats
        stats.ndecomp = stats.ndecomp + ite
        stats.nfeval = stats.nfeval + ite

        tt = tt + dt
        nt = nt + 1
        y[nt] = y1
        T[nt] = tt
        y0 = y1
        t0 = tt

    y = y[0:nt + 1]
    T = T[0:nt + 1]
    stats.nstep = nt

    return daesol(T, y, stats=stats)

==> solverz-0.1.0.tar.gz: solverz-0.1.0/Solverz/solvers/daesolver/trapezoidal.py <==
from Solverz.solvers.daesolver.utilities import *


@dae_io_parser
def implicit_trapezoid(dae: nDAE,
                       tspan: List | np.ndarray,
                       y0: np.ndarray,
                       opt: Opt = None):
    r"""
    The fixed step implicit trapezoidal method.

    Parameters
    ==========

    dae : nDAE
        Numerical DAE object.

    tspan : List | np.ndarray
        An array specifying t0 and tend

    y0 : np.ndarray
        The initial values of variables

    opt : Opt
        The solver options, including:

        - step_size: 1e-3(default)|float
            The step size

    Returns
    =======

    sol : daesol
        The daesol object.


    References
    ==========

    .. [1] https://en.wikipedia.org/wiki/Trapezoidal_rule_(differential_equations)

    """
    stats = Stats(scheme='Trapezoidal')
    if opt is None:
        opt = Opt(stats=True)
    dt = opt.step_size
    tspan = np.array(tspan)
    T_initial = tspan[0]
    T_end = tspan[-1]
    nt = 0
    tt = T_initial
    t0 = tt

    y = np.zeros((10000, y0.shape[0]))
    y0 = DaeIc(dae, y0, t0, opt.rtol)  # check and modify initial values
    y[0, :] = y0
    T = np.zeros((10000,))

    p = dae.p
    while abs(tt - T_end) > abs(dt) / 10:
        My0 = dae.M @ y0
        F0 = dae.F(t0, y0, p).copy()
        ae = nAE(lambda y_, p_: dt / 2 * (dae.F(t0 + dt, y_, p_) + F0) - dae.M @ y_ + My0,
                 lambda y_, p_: -dae.M + dt / 2 * dae.J(t0 + dt, y_, p_),
                 p)

        sol = nr_method(ae, y0, Opt(stats=True))
        y1 = sol.y
        stats.ndecomp = stats.ndecomp + sol.stats.nstep
        stats.nfeval = stats.nfeval + sol.stats.nstep

        tt = tt + dt
        nt = nt + 1
        y[nt] = y1
        T[nt] = tt
        y0 = y1
        t0 = tt

    y = y[0:nt + 1]
    T = T[0:nt + 1]
    stats.nstep = nt

    return daesol(T, y, stats=stats)
